#include "mex.h"
#include "matrix.h"
#include "interUnio_nocv.h"
#include <vector>
#include <algorithm>
#include <numeric>
using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if (2 != nrhs) {
		mexErrMsgTxt("There should be 2 arguments");
	} else if (!mxIsDouble(prhs[0]) && !mxIsInt32(prhs[0])) {
		mexErrMsgTxt("First argument should be double or int32");
	} else if (mxGetM(prhs[0]) != 4) {
		mexErrMsgTxt("First argument should have 4 rows");
	} else if (!mxIsCell(prhs[1])) {
		mexErrMsgTxt("Second argument should be cell");
	}
	mxClassID type = mxGetClassID(prhs[0]);
	
	size_t noi = mxGetNumberOfElements(prhs[1]);
	int NUM_WIN = mxGetN(prhs[0]);
	plhs[0] = mxCreateDoubleMatrix(1, NUM_WIN, mxREAL);
	plhs[1] = mxCreateDoubleMatrix(1, NUM_WIN, mxREAL);
	double *recalls = mxGetPr(plhs[0]);
	double *avgScore = mxGetPr(plhs[1]);
	for (int i = 0; i < noi; i++) {
		mxArray *mxBoxesGT = mxGetCell(prhs[1], i);
		if (mxGetClassID(mxBoxesGT) != type) {
			mexErrMsgTxt("Second argument's elements should be same type as first argument");
		} else if (mxGetM(mxBoxesGT) != 4) {
			mexErrMsgTxt("Second argument's elements should have 4 rows");
		}
		int nwo = mxGetN(mxBoxesGT);
		vector<double> score(nwo);
		vector<int> detected(nwo);
		for (int j = 0; j < NUM_WIN; j++) {
			if (nwo == 0)
				continue;
			for (int k = 0; k < nwo; k++) {
				double iu = 0;
				if (type == mxDOUBLE_CLASS)
					iu = interUnio(mxGetPr(prhs[0])+j*4, mxGetPr(mxBoxesGT)+k*4);
				else if (type == mxINT32_CLASS)
					iu = interUnio((int *)mxGetData(prhs[0])+j*4, (int *)mxGetData(mxBoxesGT)+k*4);
				else
					mexErrMsgTxt("Something must be wrong!");
				score[k] = max(score[k], iu);
				detected[k] = score[k] >= 0.5;
			}
			recalls[j] += (accumulate(detected.begin(), detected.end(), 0)/(double)nwo);
			avgScore[j] += (accumulate(score.begin(), score.end(), 0.)/(double)nwo);
		}
	}
	for (int i = 0; i < NUM_WIN; i++) {
		recalls[i] = recalls[i]/noi;
		avgScore[i] /= noi;
	}
}