function [hp, r_train, r_test] = hottestwindow(allwindows_train, allimages_train, allwindows_test, allimages_test, hp, classes)
% hp: window proposals
% r_train: DR on the train set
% r_test:  DR on the test set

tic;
for i = 1 : numel(allwindows_train)
    hr = 512 / allimages_train(i, 1);
    wr = 512 / allimages_train(i, 2);
    allwindows_train{i}([1 3], :) = allwindows_train{i}([1 3], :) * wr;
    allwindows_train{i}([2 4], :) = allwindows_train{i}([2 4], :) * hr;
end
for i = 1 : numel(allwindows_test)
    hr = 512 / allimages_test(i, 1);
    wr = 512 / allimages_test(i, 2);
    allwindows_test{i}([1 3], :) = allwindows_test{i}([1 3], :) * wr;
    allwindows_test{i}([2 4], :) = allwindows_test{i}([2 4], :) * hr;
end
if nargin >= 5
    r_train = evaluateRecall(int32(hp'), allwindows_train);
    r_test = evaluateRecall(int32(hp'), allwindows_test);
    return;
end

ws = round(sqrt(2) * 2 .^ (3 : 8));
hs = round(sqrt(2) * 2 .^ (3 : 8));
lxs = 0 : 4 : 512;
lys = 0 : 4 : 512;
covernum = zeros(numel(lxs) * numel(lys), numel(ws) * numel(hs));
coverratio = zeros(numel(lxs) * numel(lys), numel(ws) * numel(hs));

windownum_image = zeros(numel(allwindows_train), 1);
for i = 1 : numel(allwindows_train)
    windownum_image(i) = size(allwindows_train{i}, 2);
end
disp(sum(windownum_image));

acoverb = cell(numel(lxs) * numel(lys), numel(ws) * numel(hs));
bcovera = cell(numel(allwindows_train), 19);

p1 = zeros(numel(lxs) * numel(lys), numel(ws) * numel(hs), 4);

for i = 1 : numel(lxs)
    p1((i - 1) * numel(lys) + 1 : (i - 1) * numel(lys) + numel(lys), :, 1) = lxs(i);
end
for i = 1 : numel(lys)
    p1(i : numel(lys) : i + (numel(lxs) - 1) * numel(lys), :, 2) = lys(i);
end
for wi = 1 : numel(ws)
    for hi = 1 : numel(hs)
        p1(:, (wi - 1) * numel(hs) + hi, 3) = p1(:, (wi - 1) * numel(hs) + hi, 1) + ws(wi) - 1;
        p1(:, (wi - 1) * numel(hs) + hi, 4) = p1(:, (wi - 1) * numel(hs) + hi, 2) + hs(hi) - 1;
    end
end
p11 = p1(:, :, 1); p12 = p1(:, :, 2); p13 = p1(:, :, 3); p14 = p1(:, :, 4);
toc;

area = zeros(numel(lxs) * numel(lys), numel(ws) * numel(hs));
for wi = 1 : numel(ws)
    for hi = 1 : numel(hs)
        area(:, (wi - 1) * numel(hs) + hi) = ws(wi) * hs(hi);
    end
end

% flag0 = ones(numel(lxs) * numel(lys), numel(ws) * numel(hs));
% for wi = 1 : numel(ws)
%     for i = 1 : numel(lxs)
%         if lxs(i) + ws(wi) - 1 > 512
%             flag0((i - 1) * numel(lys) + 1 : i * numel(lys), ...
%                 (wi - 1) * numel(hs) + 1 : wi * numel(hs)) = 0;
%         end
%     end
% end
% for hi = 1 : numel(hs)
%     for i = 1 : numel(lys)
%         if lys(i) + hs(hi) - 1 > 512
%             flag0((0 : numel(lxs) - 1) * numel(lys) + i, ...
%                 (0 : numel(ws) - 1) * numel(hs) + hi) = 0;
%         end
%     end
% end
toc; tic;
hpn = 5000; hp = zeros(hpn, 4);
tcn = zeros(numel(lxs) * numel(lys), numel(ws) * numel(hs));
trainsamplenum = numel(allwindows_train);
for i = 1 : trainsamplenum
    if ~mod(i, 100)
        disp([i, 1]); toc;
    end
    
    tcn = tcn * 0;
    for j = 1 : size(allwindows_train{i}, 2)
        p2 = double(allwindows_train{i}(:, j));
        a0 = (p2(3) - p2(1) + 1) * (p2(4) - p2(2) + 1);
        flag = (area > a0 * 0.5) & (area < a0 * 2);
        [pos1, pos2] = find(flag); indices = sub2ind([numel(lxs) * numel(lys), numel(ws) * numel(hs)], pos1, pos2);
        flag = (p11(indices) <= p2(3)) & (p12(indices) <= p2(4)) & (p2(1) <= p13(indices)) & (p2(2) <= p14(indices));
        indices = indices(find(flag));

        p111 = p11(indices);
        p112 = p12(indices);
        p113 = p13(indices);
        p114 = p14(indices);

        x1 = max(p111, p2(1));
        x2 = min(p113, p2(3));
        y1 = max(p112, p2(2));
        y2 = min(p114, p2(4));

        inter = (x2 - x1 + 1) .* (y2 - y1 + 1);
        union = (p113 - p111 + 1) .* (p114 - p112 + 1) + a0 - inter;

        ratios =  inter ./ union > 0.5;
        tcn(indices) = tcn(indices) + ratios;
        pos = find(ratios); pos = indices(pos);
        [pos11, pos12] = ind2sub([numel(lxs) * numel(lys), numel(ws) * numel(hs)], pos);
        bcovera{i, j} = [pos11, pos12];
        for k = 1 : numel(pos11)
            acoverb{pos11(k), pos12(k)} = [acoverb{pos11(k), pos12(k)}; [i, j]];
        end
    end
    if size(allwindows_train{i}, 2)
        coverratio = coverratio + tcn / size(allwindows_train{i}, 2);
    end
    covernum = covernum + tcn;
end

for hpi = 1 : hpn
    [tv1, td1] = max(coverratio, [], 1); [tv2, td2] = max(tv1); d2 = td2; d1 = td1(d2);
    hp(hpi, :) = p1(d1, d2, :);
    acoverb_single = acoverb{d1, d2};
    for i = 1 : size(acoverb_single, 1)
        bcovera_single = bcovera{acoverb_single(i, 1), acoverb_single(i, 2)};
        for j = 1 : size(bcovera_single, 1)
            coverratio(bcovera_single(j, 1), bcovera_single(j, 2)) = ...
                coverratio(bcovera_single(j, 1), bcovera_single(j, 2)) - 1 / windownum_image(acoverb_single(i, 1));
        end
    end
    coverratio(d1, d2) = 0;
end
toc;

r_train = evaluateRecall(int32(hp'), allwindows_train);
r_test = evaluateRecall(int32(hp'), allwindows_test);
toc;

end
