#include <opencv2/opencv.hpp>
#include "mex.h"
#include "matrix.h"
using namespace cv;

#define _S(str) ((str).c_str())

void loadBox(FileNode &fn, vector<Vec4i> &boxes)
{
	string isDifficult;
	fn["difficult"] >> isDifficult;
	if (isDifficult == "1")
		return;
	string strXmin, strYmin, strXmax, strYmax;
	fn["bndbox"]["xmin"] >> strXmin;
	fn["bndbox"]["ymin"] >> strYmin;
	fn["bndbox"]["xmax"] >> strXmax;
	fn["bndbox"]["ymax"] >> strYmax;
	boxes.push_back(Vec4i(atoi(_S(strXmin)), atoi(_S(strYmin)), atoi(_S(strXmax)), atoi(_S(strYmax))));
}

bool loadBBoxes(const char *nameNE, vector<Vec4i> &boxes)
{
	string fName = string(nameNE);
	FileStorage fs(fName, FileStorage::READ);
	if (!fs.isOpened())
		mexErrMsgTxt(_S(format("Annotation file %s doesn't exist!", nameNE)));
	FileNode fn = fs["annotation"]["object"];
	boxes.clear();
	if (fn.isSeq()) {
		for (FileNodeIterator it = fn.begin(), it_end = fn.end(); it != it_end; it++) {
			loadBox(*it, boxes);
		}
	} else
		loadBox(fn, boxes);
	return true;
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if (1 != nrhs) {
		mexErrMsgTxt("There should be only one argument");
	} else if (!mxIsChar(prhs[0])) {
		mexErrMsgTxt("The argument should be a file path");
	} /* else if (1 != nlhs) {
		mexErrMsgTxt("There should be only one output");
	} */
	vector<Vec4i> boxes;
	char *fileName = mxArrayToString(prhs[0]);
	if (fileName == NULL)
		mexErrMsgTxt("Could not convert input to string");
	loadBBoxes(fileName, boxes);
	plhs[0] = mxCreateNumericMatrix(4, boxes.size(), mxINT32_CLASS, mxREAL);
	int *iter = (int *)mxGetData(plhs[0]);
	for (int i = 0; i < boxes.size(); i++) for (int j = 0; j < 4; j++) {
		*iter = boxes[i][j];
		iter++;
	}
	mxFree(fileName);
}