#include "StdAfx.h"
#include "Objectness.h"
#include "CmShow.h"

#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
void print_null(const char *s) {}
const char* Objectness::_clrName[3] = {"MAXBGR", "HSV", "I"};
const int CN = 21; // Color Number 
const char* COLORs[CN] = {"'k'", "'b'", "'g'", "'r'", "'c'", "'m'", "'y'",
	"':k'", "':b'", "':g'", "':r'", "':c'", "':m'", "':y'", 
	"'--k'", "'--b'", "'--g'", "'--r'", "'--c'", "'--m'", "'--y'"
};

//#define PRELOAD_IMGS   // Uncomment this line to remove counting times of image reading. Warning: much more memory is required

// base for window size quantization, R orientation channels, and feature window size (_W, _W)
Objectness::Objectness(DataSetVOC &voc, double base, int W, int NSS, int seed)
	: _voc(voc)
	, _base(base)							//_base = 2
	, _W(W)									//_W = 8
	, _NSS(NSS)								//_NSS = 2
	, _logBase(log(_base))					//_logBase = log(2)_base = 2
	, _minT(cvCeil(log(10.)/_logBase))		//_minT = 4
	, _maxT(cvCeil(log(500.)/_logBase))		//_maxT = 9
	, _numT(_maxT - _minT + 1)				//_numT = 6
	, _Clr(MAXBGR)							//BGR
	, yourseed(seed)
{
	setColorSpace(_Clr);
}

Objectness::~Objectness(void)
{
}

void Objectness::setColorSpace(int clr)
{
	_Clr = clr;
	_modelName = _voc.resDir + format("ObjNessB%gW%d%s", _base, _W, _clrName[_Clr]);
	_trainDirSI = _voc.localDir + format("TrainS1B%gW%d%s/", _base, _W, _clrName[_Clr]);
	_bbResDir = _voc.resDir + format("BBoxesB%gW%d%s/", _base, _W, _clrName[_Clr]);
}

int Objectness::loadTrainedModel(string modelName) // Return -1, 0, or 1 if partial, none, or all loaded
{
	if (modelName.size() == 0)
		modelName = _modelName;
	CStr s2 = modelName + ".wS2", sI = modelName + ".idx";
	Mat filters1f, reW1f, idx1i, show3u;
	if (!matRead(sI, idx1i)){
		printf("Can't load size types: %s\n", _S(sI));
		return 0;
	}

	_svmSzIdxs = idx1i;
	CV_Assert(_svmSzIdxs.size() > 1 );

	if (!matRead(s2, _svmReW1f) || _svmReW1f.size() != Size(2, _svmSzIdxs.size())){
		_svmReW1f = Mat();
		printf("Model of Stage II loading failed, but we'll continue...\n");
		return -1;
	}

	return 1;
}

void Objectness::predictBBoxSI(CMat &img3u, ValStructVec<float, Vec4i> &valBoxes, vecI &sz, int NUM_WIN_PSZ, bool fast)
{
	const int numSz = _svmSzIdxs.size();
	const int imgW = img3u.cols, imgH = img3u.rows;
	valBoxes.reserve(10000);
	sz.clear(); sz.reserve(10000);
	for (int ir = numSz - 1; ir >= 0; ir--){
		int r = _svmSzIdxs[ir];
		int height = cvRound(pow(_base, r/_numT + _minT)), width = cvRound(pow(_base, r%_numT + _minT));
		if (height > imgH * _base || width > imgW * _base)
			continue;

		height = min(height, imgH), width = min(width, imgW);
		
		Mat matchCost1f(cvRound(_W*imgH*1.0/height) - 7, cvRound(_W*imgW*1.0/width) - 7, CV_32F);

		srand(yourseed);
		for (int i = 0; i < matchCost1f.rows; i++) {
			for (int j = 0; j < matchCost1f.cols; j++) {
				matchCost1f.at<float>(i, j) = (float)((rand() % 100) / 100.0);
			}
		}

		ValStructVec<float, Point> matchCost;
		nonMaxSup(matchCost1f, matchCost, _NSS, NUM_WIN_PSZ, fast);

		double ratioX = width/_W, ratioY = height/_W;
		int iMax = min(matchCost.size(), NUM_WIN_PSZ);
		for (int i = 0; i < iMax; i++){
			float mVal = matchCost(i);
			Point pnt = matchCost[i];
			Vec4i box(cvRound(pnt.x * ratioX), cvRound(pnt.y*ratioY));
			box[2] = cvRound(min(box[0] + width, imgW));
			box[3] = cvRound(min(box[1] + height, imgH));
			box[0] ++;
			box[1] ++;
			valBoxes.pushBack(mVal, box); 
			sz.push_back(ir);
		}
	}
}

void Objectness::predictBBoxSII(ValStructVec<float, Vec4i> &valBoxes, const vecI &sz)
{
	int numI = valBoxes.size();
	for (int i = 0; i < numI; i++){
		const float* svmIIw = _svmReW1f.ptr<float>(sz[i]);
		valBoxes(i) = valBoxes(i) * svmIIw[0] + svmIIw[1]; 
	}
	valBoxes.sort();
}

// Get potential bounding boxes, each of which is represented by a Vec4i for (minX, minY, maxX, maxY).
// The trained model should be prepared before calling this function: loadTrainedModel() or trainStageI() + trainStageII().
// Use numDet to control the final number of proposed bounding boxes, and number of per size (scale and aspect ratio)
void Objectness::getObjBndBoxes(CMat &img3u, ValStructVec<float, Vec4i> &valBoxes, int numDetPerSize)
{
	vecI sz;
	predictBBoxSI(img3u, valBoxes, sz, numDetPerSize, false);
	predictBBoxSII(valBoxes, sz);
	return;
}

void Objectness::nonMaxSup(CMat &matchCost1f, ValStructVec<float, Point> &matchCost, int NSS, int maxPoint, bool fast)
{
	const int _h = matchCost1f.rows, _w = matchCost1f.cols;
	Mat isMax1u = Mat::ones(_h, _w, CV_8U), costSmooth1f;
	ValStructVec<float, Point> valPnt;
	matchCost.reserve(_h * _w);
	valPnt.reserve(_h * _w);
	if (fast){
		blur(matchCost1f, costSmooth1f, Size(3, 3));
		for (int r = 0; r < _h; r++){
			const float* d = matchCost1f.ptr<float>(r);
			const float* ds = costSmooth1f.ptr<float>(r);
			for (int c = 0; c < _w; c++)
				if (d[c] >= ds[c])
					valPnt.pushBack(d[c], Point(c, r));
		}
	}
	else{
		for (int r = 0; r < _h; r++){
			const float* d = matchCost1f.ptr<float>(r);
			for (int c = 0; c < _w; c++)
				valPnt.pushBack(d[c], Point(c, r));
		}
	}

	valPnt.sort();
	for (int i = 0; i < valPnt.size(); i++){
		Point &pnt = valPnt[i];
		if (isMax1u.at<byte>(pnt)){
			matchCost.pushBack(valPnt(i), pnt);
			for (int dy = -NSS; dy <= NSS; dy++){
				for (int dx = -NSS; dx <= NSS; dx++){
					Point neighbor = pnt + Point(dx, dy);
					if (!CHK_IND(neighbor))
						continue;
					isMax1u.at<byte>(neighbor) = false;
				}
			}
		}
		if (matchCost.size() >= maxPoint)
			return;
	}
}

void Objectness::generateTrianData()
{
	int numP = 0;

	const int NUM_TRAIN = _voc.trainNum;

	vector<vecI> szTrainP(NUM_TRAIN); // Corresponding size index. 

#pragma omp parallel for
	for (int i = 0; i < NUM_TRAIN; i++)	{
		const int NUM_GT_BOX = (int)_voc.gtTrainBoxes[i].size();
		vecI &szP = szTrainP[i];
		Mat im3u = imread(format(_S(_voc.imgPathW), _S(_voc.trainSet[i])));

		// Get positive training data
		for (int k = 0; k < NUM_GT_BOX; k++){
			const Vec4i& bbgt =  _voc.gtTrainBoxes[i][k];
			vector<Vec4i> bbs; // bounding boxes;
			vecI bbR; // Bounding box ratios
			int nS = gtBndBoxSampling(bbgt, bbs, bbR);
			numP += 2 * nS;
			for (int j = 0; j < nS; j++){
				szP.push_back(bbR[j]);
				szP.push_back(bbR[j]);
			}			
		}
	}
	
	const int NUM_R = _numT * _numT + 1;
	vecI szCount(NUM_R); // Object counts of each size (combination of scale and aspect ratio) 
	for (int i = 0; i < NUM_TRAIN; i++){
		const vecI &rP = szTrainP[i];
		for (size_t j = 0; j < rP.size(); j++)
			szCount[rP[j]]++;
	}
	vecI szActive; // Indexes of active size
	for (int r = 1; r < NUM_R; r++){
		if (szCount[r] > 50) // If only 50- positive samples at this size, ignore it.
			szActive.push_back(r-1);			
	}
	matWrite(_modelName + ".idx", Mat(szActive));
}

int Objectness::gtBndBoxSampling(const Vec4i &bbgt, vector<Vec4i> &samples, vecI &bbR)
{
	double wVal = bbgt[2] - bbgt[0] + 1, hVal = (bbgt[3] - bbgt[1]) + 1;
	wVal = log(wVal)/_logBase, hVal = log(hVal)/_logBase;
	int wMin = max((int)(wVal - 0.5), _minT), wMax = min((int)(wVal + 1.5), _maxT);
	int hMin = max((int)(hVal - 0.5), _minT), hMax = min((int)(hVal + 1.5), _maxT);
	for (int h = hMin; h <= hMax; h++) for (int w = wMin; w <= wMax; w++){
		//tLen(x) = round(2 ^ x)
		int wT = tLen(w) - 1, hT = tLen(h) - 1;
		Vec4i bb(bbgt[0], bbgt[1], bbgt[0] + wT, bbgt[1] + hT);
		if (DataSetVOC::interUnio(bb, bbgt) >= 0.5){
			samples.push_back(bb);
			bbR.push_back(sz2idx(w, h));
		}
	}
	return samples.size();
}

void Objectness::trainStateII(int numPerSz)
{
	loadTrainedModel();

	const int NUM_TRAIN = _voc.trainNum;
	vector<vecI> SZ(NUM_TRAIN), Y(NUM_TRAIN);
	vector<vecF> VAL(NUM_TRAIN);

#pragma omp parallel for
	for (int i = 0; i < _voc.trainNum; i++)	{
		const vector<Vec4i> &bbgts = _voc.gtTrainBoxes[i];
		ValStructVec<float, Vec4i> valBoxes;
		vecI &sz = SZ[i], &y = Y[i];
		vecF &val = VAL[i];
		CStr imgPath = format(_S(_voc.imgPathW), _S(_voc.trainSet[i]));
		predictBBoxSI(imread(imgPath), valBoxes, sz, numPerSz, false);

		const int num = valBoxes.size();
		CV_Assert(sz.size() == num);
		y.resize(num), val.resize(num);
		int positiveN = 0, negativeN = 0; 
		for (int j = 0; j < num; j++){
			Vec4i bb = valBoxes[j];
			val[j] = valBoxes(j);
			y[j] = maxIntUnion(bb, bbgts) >= 0.5 ? 1 : -1;
			positiveN += y[j];
		}
		negativeN = (num - positiveN) / 2;
		positiveN = num - negativeN;
		//printf("%d, %d\n", positiveN, negativeN);
	}

	const int NUM_SZ = _svmSzIdxs.size();
	const int maxTrainNum = 100000;
	vector<vecM> rXP(NUM_SZ), rXN(NUM_SZ);
	for (int r = 0; r < NUM_SZ; r++){
		rXP[r].reserve(maxTrainNum);
		rXN[r].reserve(1000000);
	}
	for (int i = 0; i < NUM_TRAIN; i++){
		const vecI &sz = SZ[i], &y = Y[i];
		vecF &val = VAL[i];
		int num = sz.size();
		for (int j = 0; j < num; j++){
			int r = sz[j];
			CV_Assert(r >= 0 && r < NUM_SZ);
			if (y[j] == 1)
				rXP[r].push_back(Mat(1, 1, CV_32F, &val[j]));
			else 
				rXN[r].push_back(Mat(1, 1, CV_32F, &val[j]));
		}
	}

	Mat wMat(NUM_SZ, 2, CV_32F);
	for (int i = 0; i < NUM_SZ; i++){
		vecM &xP = rXP[i], &xN = rXN[i];

		//If no positive exists, then choose the first ten samples as positive ones, just to avoid program crashing
		if (xP.size() == 0) {
			for (int k = 0; k < 10; k++) {
				xP.push_back(Mat(1, 1, CV_32F, xN[k].at<float>(1, 1)));
			}
		}

		if (xP.size() < 10 || xN.size() < 10)
			printf("Warning %s:%d not enough training sample for r[%d] = %d. P = %d, N = %d\n", __FILE__, __LINE__, i, _svmSzIdxs[i], xP.size(), xN.size());	
		for (size_t k = 0; k < xP.size(); k++)
			CV_Assert(xP[k].size() == Size(1, 1) && xP[k].type() == CV_32F);

		Mat wr = trainSVM(xP, xN, L1R_L2LOSS_SVC, 100, 1);
		CV_Assert(wr.size() == Size(2, 1));
		wr.copyTo(wMat.row(i));
	}

	matWrite(_modelName + ".wS2", wMat);
	_svmReW1f = wMat;
}

void Objectness::meanStdDev(CMat &data1f, Mat &mean1f, Mat &stdDev1f)
{
	const int DIM = data1f.cols, NUM = data1f.rows;
	mean1f = Mat::zeros(1, DIM, CV_32F), stdDev1f = Mat::zeros(1, DIM, CV_32F);
	for (int i = 0; i < NUM; i++)
		mean1f += data1f.row(i);
	mean1f /= NUM;
	for (int i = 0; i < NUM; i++){
		Mat tmp;
		pow(data1f.row(i) - mean1f, 2, tmp);
		stdDev1f += tmp;
	}
	pow(stdDev1f/NUM, 0.5, stdDev1f);
}

vecD Objectness::getVector(const Mat &_t1f)
{
	Mat t1f;
	_t1f.convertTo(t1f, CV_64F);
	return (vecD)(t1f.reshape(1, 1));
}

void Objectness::illustrate()
{
	Mat xP1f, xN1f;
	CV_Assert(matRead(_modelName + ".xP", xP1f) && matRead(_modelName + ".xN", xN1f));
	CV_Assert(xP1f.cols == xN1f.cols && xP1f.cols == _W*_W && xP1f.type() == CV_32F && xN1f.type() == CV_32F);
	Mat meanP,  meanN, stdDevP, stdDevN;
	meanStdDev(xP1f, meanP, stdDevP);
	meanStdDev(xN1f, meanN, stdDevN);
	Mat meanV(_W, _W*2, CV_32F), stdDev(_W, _W*2, CV_32F);
	meanP.reshape(1, _W).copyTo(meanV.colRange(0, _W));
	meanN.reshape(1, _W).copyTo(meanV.colRange(_W, _W*2));
	stdDevP.reshape(1, _W).copyTo(stdDev.colRange(0, _W));
	stdDevN.reshape(1, _W).copyTo(stdDev.colRange(_W, _W*2));
	normalize(meanV, meanV, 0, 255, NORM_MINMAX, CV_8U);
	CmShow::showTinyMat(_voc.resDir + "PosNeg.png", meanV);
	
	FILE* f = fopen(_S(_voc.resDir + "PosNeg.m"), "w"); 
	CV_Assert(f != NULL);
	fprintf(f, "figure(1);\n\n");
	PrintVector(f, getVector(meanP), "MeanP");
	PrintVector(f, getVector(meanN), "MeanN");
	PrintVector(f, getVector(stdDevP), "StdDevP");
	PrintVector(f, getVector(stdDevN), "StdDevN");
	fprintf(f, "hold on;\nerrorbar(MeanP, StdDevP, 'r');\nerrorbar(MeanN, StdDevN, 'g');\nhold off;");
	fclose(f);	
}

// Training SVM with feature vector X and label Y. 
// Each row of X is a feature vector, with corresponding label in Y.
// Return a CV_32F weight Mat
Mat Objectness::trainSVM(CMat &X1f, const vecI &Y, int sT, double C, double bias, double eps)
{
	// Set SVM parameters
	parameter param; {
		param.solver_type = sT; // L2R_L2LOSS_SVC_DUAL;
		param.C = C;
		param.eps = eps; // see setting below
		param.p = 0.1;
		param.nr_weight = 0;
		param.weight_label = NULL;
		param.weight = NULL;
		set_print_string_function(print_null);
		CV_Assert(X1f.rows == Y.size() && X1f.type() == CV_32F);
	}

	// Initialize a problem
	feature_node *x_space = NULL;
	problem prob;{
		prob.l = X1f.rows;
		prob.bias = bias;
		prob.y = Malloc(double, prob.l);
		prob.x = Malloc(feature_node*, prob.l);
		const int DIM_FEA = X1f.cols;
		prob.n = DIM_FEA + (bias >= 0 ? 1 : 0);
		x_space = Malloc(feature_node, (prob.n + 1) * prob.l);
		int j = 0;
		for (int i = 0; i < prob.l; i++){
			prob.y[i] = Y[i];
			prob.x[i] = &x_space[j];
			const float* xData = X1f.ptr<float>(i);
			for (int k = 0; k < DIM_FEA; k++){
				x_space[j].index = k + 1;
				x_space[j++].value = xData[k];
			}
			if (bias >= 0){
				x_space[j].index = prob.n;
				x_space[j++].value = bias;
			}
			x_space[j++].index = -1;
		}
		CV_Assert(j == (prob.n + 1) * prob.l);
	}

	// Training SVM for current problem
	const char*  error_msg = check_parameter(&prob, &param);
	if(error_msg){
		fprintf(stderr,"ERROR: %s\n",error_msg);
		exit(1);
	}
	model *svmModel = train(&prob, &param);
	Mat wMat(1, prob.n, CV_64F, svmModel->w);
	wMat.convertTo(wMat, CV_32F);
	free_and_destroy_model(&svmModel);
	destroy_param(&param);
	free(prob.y);
	free(prob.x);
	free(x_space);
	return wMat;
}

// pX1f, nX1f are positive and negative training samples, each is a row vector 
Mat Objectness::trainSVM(const vector<Mat> &pX1f, const vector<Mat> &nX1f, int sT, double C, double bias, double eps, int maxTrainNum)
{
	vecI ind(nX1f.size());
	for (size_t i = 0; i < ind.size(); i++)
		ind[i] = i;
	int numP = pX1f.size(), feaDim = pX1f[0].cols;
	int totalSample = numP + nX1f.size();
	
	if (totalSample > maxTrainNum)
		random_shuffle(ind.begin(), ind.end());
	totalSample = min(totalSample, maxTrainNum);
	Mat X1f(totalSample, feaDim, CV_32F);
	vecI Y(totalSample);
	for(int i = 0; i < numP; i++){
		pX1f[i].copyTo(X1f.row(i));
		Y[i] = 1;
	}
	for (int i = numP; i < totalSample; i++){
		nX1f[ind[i - numP]].copyTo(X1f.row(i));
		Y[i] = -1;
	}
	return trainSVM(X1f, Y, sT, C, bias, eps);
}

// Get potential bounding boxes for all test images
void Objectness::getObjBndBoxesForTestsFast(vector<vector<Vec4i>> &_boxesTests, int numDetPerSize)
{
	trainObjectness(numDetPerSize);
	loadTrainedModel();

	const int TestNum = _voc.testSet.size();
	vector<ValStructVec<float, Vec4i>> boxesTests;
	boxesTests.resize(TestNum);

	for (int i = 0; i < TestNum; i++)
		boxesTests[i].reserve(10000);

	printf("Start predicting\n");
	CmTimer tm("Predict");
	tm.Start();

#pragma omp parallel for
	for (int i = 0; i < TestNum; i++)
		getObjBndBoxes(imread(format(_S(_voc.imgPathW), _S(_voc.testSet[i]))), boxesTests[i], numDetPerSize);
	
	tm.Stop();
	printf("Average time for predicting an image (%s) is %gs\n", _clrName[_Clr], tm.TimeInSeconds()/TestNum);

	_boxesTests.resize(TestNum);

	CmFile::MkDir(_bbResDir);

#pragma omp parallel for
	for (int i = 0; i < TestNum; i++){
		CStr fName = _bbResDir + _voc.testSet[i];
		ValStructVec<float, Vec4i> &boxes = boxesTests[i];
		FILE *f = fopen(_S(fName + ".txt"), "w");
		fprintf(f, "%d\n", boxes.size());
		for (size_t k = 0; k < boxes.size(); k++)
			fprintf(f, "%g, %s\n", boxes(k), _S(strVec4i(boxes[k])));
		fclose(f);

		_boxesTests[i].resize(boxesTests[i].size());
		for (int j = 0; j < boxesTests[i].size(); j++)
			_boxesTests[i][j] = boxesTests[i][j];
	}

	evaluatePerImgRecall(_boxesTests, "PerImgAll.m", 5000);
}


void Objectness::getRandomBoxes(vector<vector<Vec4i>> &boxesTests, int num)
{
	const int TestNum = _voc.testSet.size();
	boxesTests.resize(TestNum);
#pragma omp parallel for
	for (int i = 0; i < TestNum; i++){
		Mat imgs3u = imread(format(_S(_voc.imgPathW), _S(_voc.testSet[i])));
		int H = imgs3u.cols, W = imgs3u.rows;
		boxesTests[i].reserve(num);
		for (int k = 0; k < num; k++){
			int x1 = rand()%W + 1, x2 = rand()%W + 1;
			int y1 = rand()%H + 1, y2 = rand()%H + 1;
			boxesTests[i].push_back(Vec4i(min(x1, x2), min(y1, y2), max(x1, x2), max(y1, y2)));
		}
	}
	evaluatePerImgRecall(boxesTests, "PerImgAll.m", num);
}

void Objectness::evaluatePerImgRecall(const vector<vector<Vec4i>> &boxesTests, CStr &saveName, const int NUM_WIN)
{
	vecD recalls(NUM_WIN);
	vecD avgScore(NUM_WIN);
	const int TEST_NUM = _voc.testSet.size();
	for (int i = 0; i < TEST_NUM; i++){
		const vector<Vec4i> &boxesGT = _voc.gtTestBoxes[i];
		const vector<Vec4i> &boxes = boxesTests[i];
		const int gtNumCrnt = boxesGT.size();
		vecI detected(gtNumCrnt);
		vecD score(gtNumCrnt);
		double sumDetected = 0, abo = 0;
		for (int j = 0; j < NUM_WIN; j++){
			if (j >= (int)boxes.size()){
				recalls[j] += sumDetected/gtNumCrnt;
				avgScore[j] += abo/gtNumCrnt;
				continue;
			}

			for (int k = 0; k < gtNumCrnt; k++)	{
				double s = DataSetVOC::interUnio(boxes[j], boxesGT[k]);
				score[k] = max(score[k], s);
				detected[k] = score[k] >= 0.5 ? 1 : 0;
			}
			sumDetected = 0, abo = 0;
			for (int k = 0; k < gtNumCrnt; k++)	
				sumDetected += detected[k], abo += score[k];
			recalls[j] += sumDetected/gtNumCrnt;
			avgScore[j] += abo/gtNumCrnt;
		}
	}

	for (int i = 0; i < NUM_WIN; i++){
		recalls[i] /=  TEST_NUM;
		avgScore[i] /= TEST_NUM;
	}

	int idx[8] = {1, 10, 100, 1000, 2000, 3000, 4000, 5000};
	for (int i = 0; i < 8; i++){
		if (idx[i] > NUM_WIN)
			continue;
		printf("%d:%.3g,%.3g\t", idx[i], recalls[idx[i] - 1], avgScore[idx[i] - 1]);
	}
	printf("\n");
	
	FILE* f = fopen(_S(_voc.resDir + saveName), "w"); 
	CV_Assert(f != NULL);
	fprintf(f, "run('.\\PlotsCVPR14.m');\n\n");
	PrintVector(f, recalls, "DR");
	PrintVector(f, avgScore, "MABO");
	fprintf(f, "semilogx(1:%d, DR(1:%d));\nhold on;\nsemilogx(1:%d, DR(1:%d));\naxis([1, 5000, 0, 1]);\nhold off;\ngrid on;\n", NUM_WIN, NUM_WIN, NUM_WIN, NUM_WIN);
	fclose(f);	
}

void Objectness::illuTestReults(const vector<vector<Vec4i>> &boxesTests)
{
	CStr resDir = _voc.localDir + "ResIlu/";
	CmFile::MkDir(resDir);
	const int TEST_NUM = _voc.testSet.size();
	for (int i = 0; i < TEST_NUM; i++){
		const vector<Vec4i> &boxesGT = _voc.gtTestBoxes[i];
		const vector<Vec4i> &boxes = boxesTests[i];
		const int gtNumCrnt = boxesGT.size();
		CStr imgPath = format(_S(_voc.imgPathW), _S(_voc.testSet[i]));
		CStr resNameNE = CmFile::GetNameNE(imgPath);
		Mat img = imread(imgPath);
		Mat bboxMatchImg = Mat::zeros(img.size(), CV_32F);

		vecD score(gtNumCrnt);
		vector<Vec4i> bboxMatch(gtNumCrnt);
		for (int j = 0; j < boxes.size(); j++){
			const Vec4i &bb = boxes[j];
			for (int k = 0; k < gtNumCrnt; k++)	{
				double mVal = DataSetVOC::interUnio(boxes[j], boxesGT[k]);
				if (mVal < score[k])
					continue;
				score[k] = mVal;
				bboxMatch[k] = boxes[j];
			}
		}

		for (int k = 0; k < gtNumCrnt; k++){
			const Vec4i &bb = bboxMatch[k];
			rectangle(img, Point(bb[0], bb[1]), Point(bb[2], bb[3]), Scalar(0), 3);
			rectangle(img, Point(bb[0], bb[1]), Point(bb[2], bb[3]), Scalar(255, 255, 255), 2);
			rectangle(img, Point(bb[0], bb[1]), Point(bb[2], bb[3]), Scalar(0, 0, 255), 1);
		}

		imwrite(resDir + resNameNE + "_Match.jpg", img);
	}
}

void Objectness::evaluatePerClassRecall(vector<vector<Vec4i>> &boxesTests, CStr &saveName, const int WIN_NUM) 
{
	const int TEST_NUM = _voc.testSet.size(), CLS_NUM = _voc.classNames.size();
	if (boxesTests.size() != TEST_NUM){
		boxesTests.resize(TEST_NUM);
		for (int i = 0; i < TEST_NUM; i++){
			Mat boxes;

			matRead(_voc.localDir + _voc.testSet[i] + ".dat", boxes);
			Vec4i* d = (Vec4i*)boxes.data;
			boxesTests[i].resize(boxes.rows, WIN_NUM);
			memcpy(&boxesTests[i][0], boxes.data, sizeof(Vec4i)*boxes.rows);
		}
	}

	for (int i = 0; i < TEST_NUM; i++)
		if ((int)boxesTests[i].size() < WIN_NUM){
			printf("%s.dat: %d, %d\n", _S(_voc.testSet[i]), boxesTests[i].size(), WIN_NUM);
			boxesTests[i].resize(WIN_NUM);
		}
	

	// #class by #win matrix for saving correct detection number and gt number
	Mat crNum1i = Mat::zeros(CLS_NUM, WIN_NUM, CV_32S);
	vecD gtNums(CLS_NUM);
	for (int i = 0; i < TEST_NUM; i++){
		const vector<Vec4i> &boxes = boxesTests[i];
		const vector<Vec4i> &boxesGT = _voc.gtTestBoxes[i];
		const vecI &clsGT = _voc.gtTestClsIdx[i];
		CV_Assert((int)boxes.size() >= WIN_NUM);
		const int gtNumCrnt = boxesGT.size();
		for (int j = 0; j < gtNumCrnt; j++){
			gtNums[clsGT[j]]++;
			double maxIntUni = 0;
			int* crNum = crNum1i.ptr<int>(clsGT[j]);
			for (int k = 0; k < WIN_NUM; k++) {
				double val = DataSetVOC::interUnio(boxes[k], boxesGT[j]);
				maxIntUni = max(maxIntUni, val);
				crNum[k] += maxIntUni >= 0.5 ? 1 : 0;
			}
		}
	}

	FILE* f = fopen(_S(_voc.resDir + saveName), "w");
	{
		CV_Assert(f != NULL);
		fprintf(f, "figure(1);\nhold on;\n\n\n");
		vecD val(WIN_NUM), recallObjs(WIN_NUM), recallClss(WIN_NUM);
		for (int i = 0; i < WIN_NUM; i++)
			val[i] = i;
		PrintVector(f, gtNums, "GtNum");
		PrintVector(f, val, "WinNum");
		fprintf(f, "\n");
		string leglendStr("legend(");
		double sumObjs = 0;
		for (int c = 0; c < CLS_NUM; c++){
			sumObjs += gtNums[c];
			memset(&val[0], 0, sizeof(double)*WIN_NUM);
			int* crNum = crNum1i.ptr<int>(c);
			for (int i = 0; i < WIN_NUM; i++){
				val[i] = crNum[i]/(gtNums[c] + 1e-200);
				recallClss[i] += val[i];
				recallObjs[i] += crNum[i];
			}
			CStr className = _voc.classNames[c];
			PrintVector(f, val, className);
			fprintf(f, "plot(WinNum, %s, %s, 'linewidth', 2);\n", _S(className), COLORs[c % CN]);
			leglendStr += format("'%s', ", _S(className));
		}
		for (int i = 0; i < WIN_NUM; i++){
			recallClss[i] /= CLS_NUM;
			recallObjs[i] /= sumObjs;
		}
		PrintVector(f, recallClss, "class");
		fprintf(f, "plot(WinNum, %s, %s, 'linewidth', 2);\n", "class", COLORs[CLS_NUM % CN]);
		leglendStr += format("'%s', ", "class");
		PrintVector(f, recallObjs, "objects");
		fprintf(f, "plot(WinNum, %s, %s, 'linewidth', 2);\n", "objects", COLORs[(CLS_NUM+1) % CN]);
		leglendStr += format("'%s', ", "objects");
		leglendStr.resize(leglendStr.size() - 2);
		leglendStr += ");";		
		fprintf(f, "%s\nhold off;\nxlabel('#WIN');\nylabel('Recall');\ngrid on;\naxis([0 %d 0 1]);\n", _S(leglendStr), WIN_NUM);
		fprintf(f, "[class([1,10,100,1000]);objects([1,10,100,1000])]\ntitle('%s')\n", _S(saveName));
		fclose(f);
		printf("%-70s\r", "");
	}

	evaluatePerImgRecall(boxesTests, CmFile::GetNameNE(saveName) + "_PerI.m", WIN_NUM);
}

void Objectness::PrintVector(FILE *f, const vecD &v, CStr &name)
{
	fprintf(f, "%s = [", name.c_str());
	for (size_t i = 0; i < v.size(); i++)
		fprintf(f, "%g ", v[i]);
	fprintf(f, "];\n");
}

// Write matrix to binary file
bool Objectness::matWrite(CStr& filename, CMat& _M){
	Mat M;
	_M.copyTo(M);
	FILE* file = fopen(_S(filename), "wb");
	if (file == NULL || M.empty())
		return false;
	fwrite("CmMat", sizeof(char), 5, file);
	int headData[3] = {M.cols, M.rows, M.type()};
	fwrite(headData, sizeof(int), 3, file);
	fwrite(M.data, sizeof(char), M.step * M.rows, file);
	fclose(file);
	return true;
}

// Read matrix from binary file
bool Objectness::matRead(const string& filename, Mat& _M){
	FILE* f = fopen(_S(filename), "rb");
	if (f == NULL)
		return false;
	char buf[8];
	int pre = fread(buf,sizeof(char), 5, f);
	if (strncmp(buf, "CmMat", 5) != 0)	{
		printf("Invalidate CvMat data file %s\n", _S(filename));
		return false;
	}
	int headData[3]; // Width, height, type
	fread(headData, sizeof(int), 3, f);
	Mat M(headData[1], headData[0], headData[2]);
	fread(M.data, sizeof(char), M.step * M.rows, f);
	fclose(f);
	M.copyTo(_M);
	return true;
}

void Objectness::evaluatePAMI12(CStr &saveName)
{
	const int TEST_NUM = _voc.testSet.size();
	vector<vector<Vec4i>> boxesTests(TEST_NUM);
	CStr dir = _voc.wkDir + "PAMI12/";
	const int numDet = 1853;
	for (int i = 0; i < TEST_NUM; i++){
		FILE *f = fopen(_S(dir + _voc.testSet[i] + ".txt"), "r");
		double score;
		boxesTests[i].resize(numDet);
		for (int j = 0; j < numDet; j++){
			Vec4i &v = boxesTests[i][j];
			fscanf(f, "%d %d %d %d %g\n", &v[0], &v[1], &v[2], &v[3], &score);
		}
		fclose(f);
	}
	printf("Load data finished\r");
	//evaluatePerImgRecall(boxesTests, saveName, numDet);
	evaluatePerClassRecall(boxesTests, saveName, numDet);
}

void Objectness::evaluateIJCV13(CStr &saveName)
{
	const int TEST_NUM = _voc.testSet.size();
	vector<vector<Vec4i>> boxesTests(TEST_NUM);
	CStr dir = _voc.wkDir + "IJCV13/";
	const int numDet = 10000;
	for (int i = 0; i < TEST_NUM; i++){
		FILE *f = fopen(_S(dir + _voc.testSet[i] + ".txt"), "r");
		boxesTests[i].resize(numDet);
		for (int j = 0; j < numDet; j++){
			Vec4i &v = boxesTests[i][j];
			fscanf(f, "%d, %d, %d, %d\n", &v[1], &v[0], &v[3], &v[2]);
		}
		fclose(f);
	}
	printf("Load data finished\r");
	evaluatePerImgRecall(boxesTests, saveName, numDet);
}

